# By     Dan Carr

TCmapNewColor  = function(
         dat,
         subs=1:ncol(dat),
	   cex=.9,
         font=1,
         titles=c('title1','title2'),
         titleLines = c(.82,.62),
         timeLine = .15,
         timeLineCex=cex,
         thresholds=NULL,
         probCuts=c(.2,.8),
         width=10,
         height=7.5,
         topMar = .4,
         botMar = 1.18,
         rnd=0,
         unitsLabel="Age-Adjusted Deaths per 100,000",
         percentsLabel="Percent of State-Years",
         changeText=c("Changes","Show in","New","Colors"),
         changeX=.22,
         changeY = c(.74,.58,.42,.26),
         changeCex=cex,
         changeFont=font,
         panelCol = c("#E0FFFF","#909090"),
         mapCol=c("#2090FF","#C0C0C0","#FF1010","#8040FF"),
         mapLineCol=c("#808080","black","black"),
         changeMapBackCol=c("white","#C0C0C0"),
         shift=list(minmaxLineUp=FALSE,unitsLabelDown=FALSE),
         pdfFile=NULL){

# ________________Script Sections__________________

#1  Class states by two thresholds for each time period  
#2  Page layout
#3  Colors and plotting parameters
#4  Plot title
#5  Outline panel groups
#6  Scaling for map panels
#7  Plot map text
#8  Plot change maps
#9  Plot years and maps 
#10 Emulated slider 


#1.  Class states by two thresholds for each time period_____________

dat = dat[,subs]
regionLabels=rownames(dat)
timeLabels=colnames(dat)
mat = as.matrix(dat)
matR = range(mat,na.rm=TRUE)

if(is.null(thresholds))
   thresholds = quantile(as.vector(mat),probs=probCuts,type=5,na.rm=TRUE)
breaks= c(matR[1],thresholds,matR[2])
matClass = matrix(cut(mat,breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat))
rownames(matClass) = regionLabels
matClass[is.na(matClass)]=4
polygonId = stateVBorders$st[is.na(stateVBorders$x)]

#2  Page Layout____________________________________

if(!is.null(pdfFile)){
     pdf(width=width,height=height,file=pdfFile)
   } else {
     windows(width=width,height=height)
}

topMar = topMar
bottomMar = botMar
leftMar = 0
rightMar = 0
borders=rep(.2,4)
nc = ncol(dat)
nr = 2
panels = panelLayout(nrow=nr,ncol=nc,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=1,rowSep=c(.2,rep(0,2)),  # leave space for year text, 2= no. rows 
            colSize=1)

panelRows = panelLayout(nrow=nr,ncol=1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)

colTabs  = panels$coltabs
colHalfWidth = diff(colTabs[2,])/2
rightPad = colHalfWidth
leftPad= colHalfWidth

panelShift =  panelLayout(nrow=nr,ncol=nc-1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftPad,rightMar=rightPad,
		rowSize=1,rowSep=c(.2,rep(0,2)),
            colSize=1)

panelShiftRows =  panelLayout(nrow=nr,ncol=1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftPad,rightMar=rightPad,
		rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)

panelTop  = panelLayout(nrow=1,ncol=1,borders=borders,
	       topMar=topMar,bottomMar=bottomMar,
	       leftMar=leftMar,rightMar=rightMar)
			
#3 Colors Renamed_________________

lowCol=mapCol[1]
midCol=mapCol[2]
hiCol = mapCol[3]
missCol = mapCol[4]
backCol = midCol

# Currently I assume all regions have values when including NAs
# Thus all middle row states will be filled
# There is no need for a separate background fill color
# I fill all states with midCol and later skip midCol plotting  

backLineCol = mapLineCol[1]
backLineLwd = 1
foreLineCol = mapLineCol[2]
foreLineLwd = 1
nationLineCol = mapLineCol[3]
nationLineLwd = 1

changeMapCol = changeMapBackCol[1]
changeMapLineCol = changeMapBackCol[2]

#4 Plot title____________________________________

if(!is.null(titles)){
   par(xpd=TRUE)
   panelSelect(panelTop,mar='top')
   panelScale()

  if(length(titles)==1){
     text(.5,titleLines[1],titles[1],cex=1)
  } else {
    text(.5,titleLines[1],titles[1],cex=1) 
    text(.5,titleLines[2],titles[2],cex=1)
  }
}


#5 Outline upper and lower panels__________________________

panelSelect(panelRows,1,1)
panelScale()
panelFill(col=panelCol[1])
panelOutline(col=panelCol[2])

panelSelect(panelRows,2,1)
panelScale()
panelFill(col=panelCol[1])
panelOutline(col=panelCol[2])

#6 Scaling for map panels________________________________

bndOrd = stateVBorders$st[is.na(stateVBorders$x)] # State abbrev
rxpoly = range(stateVBorders$x,na.rm=T)
rxpoly = mean(rxpoly)+ 1.06*diff(rxpoly)*c(-.5,.5)
rypoly = range(stateVBorders$y,na.rm=T)
rypoly = mean(rypoly)+ 1.08*diff(rypoly)*c(-.5,.5)

#7 Plot Map Text___________________________________________

panelSelect(panels,2,1)
panelScale()
text(changeX,changeY,changeText,
     adj=.5,cex=changeCex)

#8 Plot Change Map__________________________________

for (j in 1:(nc-1)){
   # Scale the panel
   panelSelect(panelShift,2,j)   #
   panelScale(rxpoly,rypoly)
   # panelOutline()

   # plot states in gray with white outline
   polygon(stateVBorders$x, stateVBorders$y,col=changeMapCol,
           border=changeMapLineCol) # fill states

   # changing regions 
      
   good = matClass[,j] != matClass[,j+1]
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]
      newClass = matClass[forePolyId,j+1]  
      oldClass = matClass[forePolyId,j]  
      penCol=mapCol[newClass]
      #lineCol=lineNew[oldClass]  
      polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=penCol,
             border="black") # fill states
   }
}

# 9 Plot Years and Main Maps

for(j in 1:nc){
   panelSelect(panels,1,j)
   panelScale()
   mtext(timeLabels[j],side=3,line=timeLine,cex=timeLineCex)

   panelSelect(panels,1,j)   #
   panelScale(rxpoly,rypoly)
   # plot states in gray with white outline
   polygon(stateVBorders$x, stateVBorders$y,col=backCol,border=T) # fill states
   polygon(stateVBorders$x, stateVBorders$y,density=0,
         col=backLineCol,lwd=backLineLwd) # outline states

   good = matClass[,j]==4
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(stateVBorders$st,hRegions))  
       polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=missCol,
       border=foreLineCol) # fill states
   }

   good = matClass[,j]==1
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(stateVBorders$st,hRegions))
       polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=lowCol,
       border=foreLineCol) # fill states
   }

   good= matClass[,j]==3 
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(stateVBorders$st,hRegions))
       polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=hiCol,
       border=foreLineCol) # fill states
   }

   # draw the US border
#   polygon(nationVBorders$x,nationVBorders$y,density=0,
#      col="black",lwd=borderLineLwd) # outside boundary
}

#10 Emulated slider

# Uses my functions panelLayout(), panelSelect() and panelScale()
# The slider occupies space allocated in the bottom margin 
# Sets the scale  to [0 1] x [0 1]
# Actual size depends on the window size and the panelLayout()allocation
# that create the panels argument.    

panelSelect(panels,mar='bottom')
panelScale()

# sliderX
minX = .14
lowAdjX = .20
hiAdjX = .80
maxX = .86

# sliderY
percentUnitsY = .80
percentValY = .65
thickTop = .55
thinTop = .515
thinBot = .445
thickBot = .41
valLabY = .32
minmaxLabY =.18
valUnitsY = .18


if(shift$minmaxLineUp)minmaxLabY = valLabY
if(shift$unitsLabelDown)valUnitsY=.04
# attaching and detaching lists is an experiment with
# with a seemingly simple to pass and access many named argument. 

# Thin Rectangles________________________________

# Draws thin rectangle from adjacent values to
# corresponding extreme values WHEN they are different
#
# sliderX above gives the x coordinates 
# sliderY coordinates are thinTop and thinBot give the Y coordinates

# Thick Rectangles___________________________________

# SliderX: gives the candidate X coordinates
# SliderY: thickTop and thickBot give Y Coordinates  


# Thin rectangles and color
#      If (minX < lowAdj) plot left tail in low color
#           If (lowCut < lowAdj) plot lowCut to lowAdj in mid color
#           If (hiCut < low Adj) plot hiCut to lowAdj in hi color
#
#      If (hiAdj < maxX) plot right tail in hi color
#           If (hiCut > hiAdj) plot hiAdj to hiCut in mid color
#           If (lowCut > hiAdj) plot hiAdj to lowCut in hi color
       
#  Thick rectangles and color
#       plot lowAdj to hiAdj in mid color
#  if (lowAdj < lowCut) plot lowAdj to lowCut in lowCol
#  if (hiCut < hiAdj) plot hiCut to hiAdj in highCol

#  Slider Feature label locations 
#       lower and upper cut values at computed x's and valLabY
#       minVal and maxVal at minX, maxX and minmaxLabY 
#       If min < low adjacent lowAdj at lowAdjX and minmaxLabY
#       if max > upper adjaent hiAdj at hiAdjX an minmaxlabY
#
#  Labeling Note
#    The Thick rectangle labeling takes care of 
#       when different than min max values
#       and min max values when not different.  adjacent values


# Data related values 

tmpDat = as.numeric(as.matrix(dat))
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = thresholds[1]
hiCut = thresholds[2]

sliderOutline = "black"

if(minVal < lowAdj){
      rect(minX,thinBot,lowAdjX,thinTop,col=lowCol,border=sliderOutline)
      text(minX,minmaxLabY, round(minVal,rnd),adj=.5,cex=cex)

if(lowCut < lowAdj){
   xloc = approx(c(minVal,lowAdj),c(minX,lowAdjX),lowCut)$y
         rect(xloc,thinBot,lowAdjX,thinTop,col=midCol,border=sliderOutline)
   text(xloc,minValY,round(lowCut,rnd),adj=.5,cex=cex)
         lowCutLoc=xloc # save for percent location calculation
      }
      if(hiCut < lowAdj){
   xloc = approx(c(minVal,lowAdj),c(minX,lowAdjX),hiCut)$y
         rect(xloc,thinBot,lowAdjX,thinTop,col=hiCol,border=sliderOutline)
   text(xloc,valLabY,round(hiCut,rnd),adj=.5,cex=cex)
         hiCutLoc=xloc  # save for percent location calculation
      }
}

if(maxVal > hiAdj){
      rect(hiAdjX,thinBot,maxX,thinTop,col=hiCol,border=sliderOutline)
      text(maxX,minmaxLabY, round(maxVal,rnd),adj=.5,cex=cex)
if(hiCut > hiAdj){
   xloc = approx(c(hiAdj,maxVal),c(hiAdjX,maxX),hiCut)$y
         rect(hiAdjX,thinBot,xloc,thinTop,col=midCol,border=sliderOutline)
   text(xloc,valLabY,round(hiCut,rnd),adj=.5,cex=cex)
         hiCutLoc=xloc  # save for percent location calculation
      }
if(lowCut > hiAdj){
   xloc = approx(c(hiAdj,maxVal),c(hiAdjX,maxX),lowCut)$y
         rect(hiAdjX,thinBot,xloc,thinTop,col=lowCol,border=sliderOutline)
   text(xloc,valLabY,round(lowCut,rnd),adj=.5,cex=cex)
         lowCutLoc=xloc # save for percent location calculation
      }
}

# large Rectangles
  if(lowAdj == minVal)lowAdjX=minX
  if(hiAdj == maxVal)hiAdjX=maxX
  rect(lowAdjX,thickBot,hiAdjX,thickTop,col=midCol,border=sliderOutline)
  text(lowAdjX,minmaxLabY,round(lowAdj,rnd),adj=.5,cex=cex)
  text(hiAdjX,minmaxLabY,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(lowAdjX,hiAdjX),lowCut)$y
   rect(lowAdjX,thickBot,xloc,thickTop,col=lowCol,border=sliderOutline)
   text(xloc,valLabY,round(lowCut,rnd),adj=.5,cex=cex)   
   lowCutLoc = xloc  # save for percent location calculation
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(lowAdjX,hiAdjX),hiCut)$y
   rect(xloc,thickBot,hiAdjX,thickTop,col=hiCol,border=sliderOutline)
   text(xloc,valLabY,round(hiCut,rnd),adj=.5,cex=cex)
   hiCutLoc=xloc # save for percent location calculation
}

# percents
leftEdge = ifelse(minVal < lowAdj,minX,lowAdjX)
rightEdge = ifelse(maxVal > hiAdj,maxX,hiAdjX)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(percentValY,3),
     paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,percentUnitsY,percentsLabel,cex=cex,adj=.5)
text(.5,valUnitsY,unitsLabel,cex=cex,adj=.5)

if(!is.null(pdfFile))dev.off()
}


