# File          Fig6_3.r
#               World Caricature for Yield Spread
#
# By            Dan Carr
# Date          June 17, 2009; Edited May 1, 2010

# Sections:     1. Graphics parameters
#               2. Read, select, and difference yield spread data 
#               3. Read continent polygons and prepare caricature
#               4. Read country points and modify correspondingly
#
#               5. Select data columns and get labels
#               6. Set Slider Threshholds for Both Sliders
#               7. Compute Classes For Yield Spread and First Differences
#               8. Page Layout
#
#               9. More color and plotting parameters
#              10. Plot title
#              11. Row and Column Labels
#              12. Map Yield Spread Points
# 
#              13. Map Difference in Spread Map Points
#              14. Yield Spread Emulated Slider
#              15. Difference in Spread Emulated Slider
#              16. Close Graphics device 
#    
# 1. Graphics parameters 
titles=NULL
titleLines = c(.9,.33)
width=6.5
height=8
topMar=.4
botMar=.6
rnd=2
cex=1.2
cexT=.8
timeLine = .3
sliderPercents="Percent of State Years"
borderCol="#909090"
shift=TRUE
pdfFile="Fig6_3.pdf"
font2=2
colClass=c("#008837","#A8D8B7","#FFFFFF","#C2A5CF",
                   "#7B3294")

# 2. Read, select, and difference yield spread data 

intSpread  = read.csv(file="WorldIntRatesSpreadJuneUse.csv",
            header=TRUE, stringsAsFactors=FALSE,row.names=1)
#colnames(intSpread)

# Focus on 4 months with about one month gap between each
intSpreadDat = intSpread[,c(28,30,32,34)]
colnames(intSpreadDat) = c('Aug 22 2008','Oct 16 2008',
                        'Dec 11 2008','Feb 05 2009')

# 3. Read continent polygons and prepare caricature

continentPoly = read.table('ContinentPolyLatLong.txt',
                colClasses=c('numeric','numeric'))

x = continentPoly[,1] 
y = continentPoly[,2]

# move New Zealand 
pick = x>160 & y < 20 
x = ifelse(pick, x-10,x)
y = ifelse(pick, y+6,y)

# Remove Slice of Atlantic Ocean
x = ifelse(x>-21,x-23,x)

rx = range(x,na.rm=T)
rx
ry = range(y,na.rm=T)
rx[1]= -113
rx[2]= 149
ry[1] = -42
ry[2]=67

#plot(rx,ry,type='n',axes=FALSE)
#polygon(x,y)
#box()

# 4. Read country points and modify correspondingly

country  = read.csv(file="CountryLatLong.csv",header=TRUE,
           stringsAsFactors=FALSE,row.names=1)

px= country[,1]
py= country[,2]


# move New Zealand 
pick = px>160 & py < 20 
px = ifelse(pick, px-10,px)
py = ifelse(pick, py+6,py)

# Remove Slice of Atlantic Ocean
px = ifelse(px>-21,px-20,px)

# 5. Select data columns and get labels________________________

subs=1:4
dat =intSpreadDat[,subs]
nr=ncol(dat)  
regionLabels=rownames(dat)
timeLabels=colnames(dat)

#6. Set Slider Threshholds for Both Sliders____________________

threshholds=c(1,2)
threshholds1=c(0,1)

#7. Compute Classes For Yield Spread and First Differences_______

mat = as.matrix(dat)
matR = range(mat,na.rm=TRUE)
if(is.null(threshholds))
   threshholds = quantile(as.vector(mat),probs=probCuts,type=5,na.rm=TRUE)
breaks= c(matR[1],threshholds,matR[2])
matClass = matrix(cut(mat,breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat))
rownames(matClass) = regionLabels

# mat1
mat1= t(diff(t(mat)))
mat1R = range(mat1,na.rm=TRUE)
if(is.null(threshholds1))
  threshholds1 = quantile(as.vector(mat1),probs=probCuts1,type=5,na.rm=TRUE)
breaks1= c(mat1R[1],threshholds1,mat1R[2])
mat1Class = matrix(cut(mat1,breaks=breaks1,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat1))
rownames(mat1Class) = regionLabels

#8 Page Layout____________________________________

if(!is.null(pdfFile)){
     pdf(width=width,height=height,file=pdfFile)
   } else {
     windows(width=width,height=height)
}

bottomMar = 1
leftMar = .3
rightMar = 0
#borders=c(1,.2,.2,.2)
borders=rep(.1,4)
nc = 2 
nr = ncol(dat)

panels = panelLayout(nrow=nr,ncol=2,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
            colSep=c(0,.00,0),
            colSize=1)

rowTabs  = panels$rowtabs
rowHalfWidth = diff(rowTabs[2,])/2
topPad = rowHalfWidth
botPad= rowHalfWidth


botPadShift = .7750
panelShift =  panelLayout(nrow=nr,ncol=2,borders=borders,
	        topMar=topMar+topPad,bottomMar=botPadShift,
	        leftMar=leftMar,rightMar=rightMar,
              rowSize=c(1,1,1,.7742),
              colSep=c(0,.00,0),
              colSize=1)
	
#9 Colors and plotting parameters_________________

oddCol =  "#E0FFFF"
evenCol = "#E0FFFF"
midGray = "#909090"

lightGray = "#E0E0E0"
lightYellow= "#FFFFA0"
backgroundColors = c(rep(lightGray,3),lightYellow)

foreColor = "#F00000"
foreLineCol = "#000000"
foreLineLwd = 1

lowCol = "#0080FF"
midCol = "#FFFF00"      #  "#909090"   
hiCol = "#FF1010"
penNew = c(lowCol,midCol,hiCol)

lowCol1 = "#018571"
midCol1 = "white" # "#FF8000"      
hiCol1 =  "#A050FF" # "#A6611A"
penNew1 = c(lowCol1,midCol1,hiCol1)

lowCol2 = "#40C040"
midCol2 = "#E0E0E0"      #  "#909090"   
hiCol2 = "#A050E0"
penNew2 = c(lowCol2,midCol2,hiCol2)

penLeave = c("#80B0FF","#B0B0B0","#FFA0A0")
penSame = c("#80B0FF","#B0B0B0","#FFA0A0")
	
backColor = "#D0D0D0"
backLineCol = "#FFFFFF"
backLineLwd = 1

borderLineCol = "#808080"
borderLineLwd = 1
dcex = 1.15
cex = .8

lightLineCol= "#B0B0B0"

#10  Plot title____________________________________

if(!is.null(titles)){
   par(xpd=TRUE)
   panelSelect(panels,mar='top')
   panelScale()
 if(length(titles)==1){
     text(.5,titleLines[1],titles[1],cex=1)
  } else {
    text(.5,titleLines[1],titles[1],cex=1) 
    text(.5,titleLines[2],titles[2],cex=1)
  }
}

# 11. Row and Column Labels___________________________________

# Row labels
for(i in 1:nr){
   panelSelect(panels,i,1)
   panelScale()
   mtext(substring(timeLabels[i],1,3),line=.4,at=.61,side=2,las=1,cex=cex)   #time Label
   loc =ifelse(i<4,5,6)
    dateline =ifelse(i<4,.6,.73)
    mtext(substring(timeLabels[i],loc,6),line=dateline,at=.50,side=2,las=1,cex=cex)   #time Label
    mtext(substring(timeLabels[i],8,11),line=.2,at=.39,side=2,las=1,cex=cex)
}

# Column Labels

panelSelect(panels,1,1)
panelScale()
mtext("Ten Year Minus Two Year Yield Spread",side=3,line=timeLine,cex=cex,font=font2)   #time Label
panelSelect(panelShift,1,2)
panelScale()
mtext("First Difference of Spreads",side=3,line=timeLine,cex=cex,font=font2)

#12. Map Yield Spread Points__________________________ 

for(i in 1:nr){
   panelSelect(panels,i,1)   #
   panelScale(rx,ry)
   panelFill(col="#F0F8FF")
   par(xpd=FALSE)
   polygon(x,y,density=-1,col="#E0E0E0",border="#808080")
   panelOutline()
   points(px,py,pch=21,bg=penNew[matClass[,i]],col="black",cex=dcex) 
}

#13.  Map Difference in Spread Map Points

for(i in 1:(nr-1)){
   panelSelect(panelShift,i,2)   #
   panelScale(rx,ry)
   panelFill(col="#F0F8FF")
   par(xpd=FALSE)
   polygon(x,y,density=-1,col="#E0E0E0",border="#808080")
   panelOutline()
   points(px,py,pch=21,bg=penNew1[mat1Class[,i]],col="black",cex=dcex) 
}

#14 Yield Spread Emulated Slider_______________________________

# bottom Margin scaling
# 0 .4 3.125 3.285 6.010
# 
#   1 slider unit = 2.725
# .4/2.725 = .1468
# 6.010/2.725 = 2.2055
# scale 1 = [-.1468  2.2055-.1468]
#           [-.1468  2.0587]
# scale 2 = [0  2.2055]
#           [-1.2055   1]
tmpDat = as.numeric(mat)	
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = threshholds[1]
hiCut = threshholds[2]

panelSelect(panels,mar='bot')
panelScale(c(-.1468,2.0587),c(0,1))

# Slider Design Y values

yInc = .065

slTitleY = .79 + yInc  
slLabY = .10 + yInc 
slThickHi = .53 + yInc  #.51
slThickLow = .37 + yInc #.35
slThinHi = .47 + yInc
slThinLow = .39 + yInc

slPer = .62 + yInc 
slThresh = .28 + yInc
# slMinMax = .17 + yInc 
slMinMax = .28 + yInc 
pyThick = c(slThickLow,slThickHi,slThickHi,slThickLow,NA)
pyThin  = c(slThinLow,slThinHi,slThinHi,slThinLow,NA)

if(minVal < lowAdj){
	px = c(.08,.08,.20,.20,NA)
	polygon(px,pyThin,col=lowCol,border="black")
      text(.08,if(shift)slMinMax else slThresh,round(minVal,rnd),adj=.5,cex=cex)
	if(lowCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.08,.20),lowCut)$y
         lowCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=midCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.08,.20),hiCut)$y
         hiCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=hiCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }

}

if(maxVal > hiAdj){
	px = c(.80,.80,.92,.92,NA)
	polygon(px,pyThin,col=hiCol,border="black")
      text(.92,if(shift) slMinMax else slThresh,round(maxVal,rnd),adj=.5,cex=cex)
	if(hiCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.92),hiCut)$y
         hiCutLoc=xloc
	   px = c(.92,.92,xloc,xloc,NA)
	   polygon(px,pyThin,col=midCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
	if(lowCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.92),lowCut)$y
         lowCutLoc=xloc
	   px = c(.80,.80,xloc,xloc,NA)
	   polygon(px,pyThin,col=lowCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) slMinMax else slThresh,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) slMinMax else slThresh,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)   
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents	
leftEdge = ifelse(minVal < lowAdj,.08,.20)
rightEdge = ifelse(maxVal > hiAdj,.92,.80)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(slPer,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,slTitleY,"Percent of Nation-Months",cex=cex,adj=.5)
text(.5,slLabY,"Yield Spread in Percent",cex=cex,adj=.5)
#par(xpd=TRUE)
#lines(c(.1,.9),c(0,0),col="#E0E0E0")


#15 Difference in Spread Emulated Slider 

tmpDat = as.numeric(mat1)	
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = threshholds1[1]
hiCut = threshholds1[2]

par(xpd=TRUE)
panelSelect(panelShift,4,2)
panelScale(rx=c(0,1),ry=c(0,1))

pyThick = c(slThickLow,slThickHi,slThickHi,slThickLow,NA)
pyThin  = c(slThinLow,slThinHi,slThinHi,slThinLow,NA)

if(minVal < lowAdj){
	px = c(.08,.08,.20,.20,NA)
	polygon(px,pyThin,col=lowCol1,border="black")
      text(.08,if(shift)slMinMax else slThresh,round(minVal,rnd),adj=.5,cex=cex)
	if(lowCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.08,.20),lowCut)$y
         lowCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=midCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.08,.20),hiCut)$y
         hiCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=hiCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
}

if(maxVal > hiAdj){
	px = c(.80,.80,.92,.92,NA)
	polygon(px,pyThin,col=hiCol1,border="black")
      text(.92,if(shift) slMinMax else slThresh,round(maxVal,rnd),adj=.5,cex=cex)
	if(hiCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.92),hiCut)$y
         hiCutLoc=xloc
	   px = c(.92,.92,xloc,xloc,NA)
	   polygon(px,pyThin,col=midCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
	if(lowCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.92),lowCut)$y
         lowCutLoc=xloc
	   px = c(.80,.80,xloc,xloc,NA)
	   polygon(px,pyThin,col=lowCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol1)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) slMinMax else slThresh,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) slMinMax else slThresh,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol1)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)   
}


if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol1)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents	
leftEdge = ifelse(minVal < lowAdj,.08,.20)
rightEdge = ifelse(maxVal > hiAdj,.92,.80)

classTot = table(mat1Class)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(slPer,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,slTitleY,"Percent of All First Differences",cex=cex,adj=.5)
text(.5,slLabY,"Spread Difference Over Eight Week Periods",cex=cex,adj=.5)

# 16. Close Graphics device

if(!is.null(pdfFile))dev.off()




