# File      TCmapsDiffSlides
# By        Dan Carr
# Date      May 23, 2007; Edited Slightly May 1, 2010

# For       Fig10_6
# Note 1    Sometime replace the emulated slider part
#           with more current script            
# Note 2    Sometime Add argument description

TCmapDiffSliders  = function(
         dat,
         subs=1:ncol(dat),
         titles=c('title1','title2'),
         titleLines=c(.98,.35),
         timeLine = .15,
         probCuts=c(.2,.8),
         thresholds=NULL,
         width=10,
         height=7.8,
         topMar = .4,
         rnd=0,
	   sliderLabel="Age-Adjusted Deaths per 100,000",
         rowlab=c("Rates","First Differences","Second Differences"),
         font2=1,
         cex=.6,
         cexT=.8, 
         borderCol="black",
         shift=TRUE,
         colClass=c("#008837","#A8D8B7","#FFFFFF","#C2A5CF","#7B3294"), 
         pdfFile=NULL){


# ________________Script Sections__________________

#1  Class states by two thresholds for each time period  
#2  Page layout
#3  Colors and plotting parameters
#5  Plot title
#6  Outline panel groups
#7  Scaling for map panels
#8  Plot contours
#9 Emulated slider 

#1.  Class states by two thresholds for each time period_____________

dat = dat[,subs]
regionLabels=rownames(dat)
timeLabels=colnames(dat)
mat = as.matrix(dat)
matR = range(mat,na.rm=TRUE)
thresholds = quantile(as.vector(mat),probs=probCuts,type=5,na.rm=TRUE)
breaks= c(matR[1],thresholds,matR[2])
matClass = matrix(cut(mat,breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat))
rownames(matClass) = regionLabels
#matClass[is.na(matClass)]=4


mat1= t(diff(t(mat)))
mat2 = t(diff(t(mat1)))
mat1R = range(mat1,na.rm=TRUE)
mat2R = range(mat2,na.rm=TRUE)
thresholds1 = quantile(as.vector(mat1),probs=probCuts,type=5,na.rm=TRUE)
thresholds2 = quantile(as.vector(mat2),probs=probCuts,type=5,na.rm=TRUE)
breaks1= c(mat1R[1],thresholds1,mat1R[2])
breaks2= c(mat2R[1],thresholds2,mat2R[2])
mat1Class = matrix(cut(mat1,breaks=breaks1,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat1))
mat2Class = matrix(cut(mat2,breaks=breaks2,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat2))
rownames(mat1Class) = regionLabels
rownames(mat2Class) = regionLabels

#2  Page Layout____________________________________

if(!is.null(pdfFile)){
     pdf(width=width,height=height,file=pdfFile)
   } else {
     windows(width=width,height=height)
}

bottomMar = 0
leftMar = .08
rightMar = 0
#borders=c(1,.2,.2,.2)
borders=rep(.2,4)
nc = ncol(dat)
nr=6
slideHt=.95
rowSize=c(1,1,1,rep(slideHt,3))

panels = panelLayout(nrow=nr,ncol=nc,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=rowSize,rowSep=c(.2,rep(0,nr)),
            colSize=1)


panelRows = panelLayout(nrow=nr,ncol=1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=rowSize,rowSep=c(.2,rep(0,nr)),
            colSize=1)

colTabs  = panels$coltabs
colHalfWidth = diff(colTabs[2,])/2
rightPad = colHalfWidth
leftPad= colHalfWidth


panelShift =  panelLayout(nrow=nr,ncol=nc-1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftPad,rightMar=rightPad,
		rowSize=rowSize,rowSep=c(.2,rep(0,nr)),
            colSize=1)

panelShiftRows =  panelLayout(nrow=nr,ncol=1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftPad,rightMar=rightPad,
		rowSize=rowSize,rowSep=c(.2,rep(0,nr)),
            colSize=1,colSep=c(.2,0))

	
#3 Colors and plotting parameters_________________

oddCol =  "#E0FFFF"
evenCol = "#E0FFFF"
midGray = "#909090"

lightGray = "#E0E0E0"
lightYellow= "#FFFFA0"
backgroundColors = c(rep(lightGray,3),lightYellow)

foreColor = "#F00000"
foreLineCol = "#000000"
foreLineLwd = 1

lowCol = "#0080FF"
midCol = "#E0E0E0"      #  "#909090"   
hiCol = "#FF1010"
penNew = c(lowCol,midCol,hiCol)


lowCol1 = "#018571"
midCol1 = "#E0E0E0"      #  "#909090"   
hiCol1 = "#A6611A"
penNew1 = c(lowCol1,midCol1,hiCol1)

lowCol2 = "#40C040"
midCol2 = "#E0E0E0"      #  "#909090"   
hiCol2 = "#A050E0"
penNew2 = c(lowCol2,midCol2,hiCol2)


penLeave = c("#80B0FF","#B0B0B0","#FFA0A0")
penSame = c("#80B0FF","#B0B0B0","#FFA0A0")
	
backColor = "#D0D0D0"
backLineCol = "#FFFFFF"
backLineLwd = 1

borderLineCol = "#808080"
borderLineLwd = 1
cex = cex

lightLineCol= "#B0B0B0"

#6  Plot title____________________________________

if(!is.null(titles)){
   par(xpd=TRUE)
   panelSelect(panels,mar='top')
   panelScale()
 if(length(titles)==1){
     text(.5,titleLines[1],titles[1],cex=1)
  } else {
    text(.5,titleLines[1],titles[1],cex=1) 
    text(.5,titleLines[2],titles[2],cex=1)
  }
}

#7 Outline upper and lower panels__________________________

panelSelect(panelRows,1,1)
panelScale()
panelFill(col=oddCol)
panelOutline(col=midGray)

panelSelect(panelRows,2,1)
panelScale()
panelFill(col=evenCol)
panelOutline(col=midGray)

panelSelect(panelRows,3,1)
panelScale()
panelFill(col=oddCol)
panelOutline(col=midGray)

#for(i in 4:6){
#panelSelect(panelRows,i,1)
#panelScale()
#panelOutline(col="#E0E0E0")
#}




#8 Scaling for map panels________________________________

bndOrd = stateVBorders$st[is.na(stateVBorders$x)] # State abbrev
rxpoly = range(stateVBorders$x,na.rm=T)
rxpoly = mean(rxpoly)+ 1.06*diff(rxpoly)*c(-.5,.5)
rypoly = range(stateVBorders$y,na.rm=T)
rypoly = mean(rypoly)+ 1.08*diff(rypoly)*c(-.5,.5)
# get polygon Id's
polygonId = stateVBorders$st[is.na(stateVBorders$x)]

# 9 Plot Contours___________________________________________

# Column labels
for(j in 1:nc){
   panelSelect(panels,1,j)
   panelScale()
   mtext(timeLabels[j],side=3,line=timeLine,cex=cex,font=font2)   #time Label
}

# Row Labels

for(i in 1:3){
   panelSelect(panels,i,1)
   panelScale()
   mtext(rowlab[i],side=2,line=timeLine,cex=cex,font=font2)   #time Label
}


for(j in 1:nc){
   panelSelect(panels,1,j)   #
   panelScale(rxpoly,rypoly)
   # plot states in gray with white outline
   polygon(stateVBorders$x, stateVBorders$y,col=backColor,
      border=backLineCol,lwd=backLineLwd) # outline states
   polygon(nationVBorders$x,nationVBorders$y,density=0,
      col=borderCol,lwd=borderLineLwd) # outside boundary 

   good = matClass[,j]!= 2 
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(stateVBorders$st,hRegions))
       forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
       penCol=penNew[matClass[forePolyId,j]]
       polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=penCol,border=F) # fill states
       polygon(stateVBorders$x[fore], stateVBorders$y[fore],density=0,
          col=foreLineCol, lwd=1) # outline states
   }
}

for(j in 1:(nc-1)){
   panelSelect(panelShift,2,j)   #
   panelScale(rxpoly,rypoly)
   # plot states in gray with white outline
   polygon(stateVBorders$x, stateVBorders$y,col=backColor,
      border=backLineCol,lwd=backLineLwd) # outline states
   polygon(nationVBorders$x,nationVBorders$y,density=0,
      col=borderCol,lwd=borderLineLwd) # outside boundary 

   good = mat1Class[,j] != 2 
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(stateVBorders$st,hRegions))
       forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
       penCol=penNew1[mat1Class[forePolyId,j]]
       polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=penCol,border=F) # fill states
       polygon(stateVBorders$x[fore], stateVBorders$y[fore],density=0,
          col=foreLineCol, lwd=1) # outline states
   }
}

for(j in 1:(nc-2)){
   panelSelect(panels,3,j+1)   #
   panelScale(rxpoly,rypoly)
   # plot states in gray with white outline
   polygon(stateVBorders$x, stateVBorders$y,col=backColor,
      border=backLineCol,lwd=backLineLwd) # outline states
   polygon(nationVBorders$x,nationVBorders$y,density=0,
      col=borderCol,lwd=borderLineLwd) # outside boundary 

   good = mat2Class[,j]!= 2 
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(stateVBorders$st,hRegions))
       forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
       penCol=penNew2[mat2Class[forePolyId,j]]
       polygon(stateVBorders$x[fore], stateVBorders$y[fore], col=penCol,border=F) # fill states
       polygon(stateVBorders$x[fore], stateVBorders$y[fore],density=0,
          col=foreLineCol, lwd=1) # outline states
   }
}

#10 Emulated slider1 

tmpDat = as.numeric(mat)	
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = thresholds[1]
hiCut = thresholds[2]

panelSelect(panelRows,4,1)
panelScale()

# Slider Design Y values

yInc = .065

slTitleY = .79 + yInc  
slLabY = .08 + yInc 
slThickHi = .51 + yInc
slThickLow = .35 + yInc
slThinHi = .47 + yInc
slThinLow = .39 + yInc

slPer = .62 + yInc 
slThresh = .26 + yInc
slMinMax = .15 + yInc

pyThick = c(slThickLow,slThickHi,slThickHi,slThickLow,NA)
pyThin  = c(slThinLow,slThinHi,slThinHi,slThinLow,NA)

if(minVal < lowAdj){
	px = c(.14,.14,.20,.20,NA)
	polygon(px,pyThin,col=lowCol,border="black")
      text(.14,if(shift)slMinMax else slThresh,round(minVal,rnd),adj=.5,cex=cex)
	if(lowCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),lowCut)$y
         lowCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=midCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),hiCut)$y
         hiCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=hiCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }

}

if(maxVal > hiAdj){
	px = c(.80,.80,.86,.86,NA)
	polygon(px,pyThin,col=hiCol,border="black")
      text(.86,if(shift) slMinMax else slThresh,round(maxVal,rnd),adj=.5,cex=cex)
	if(hiCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),hiCut)$y
         hiCutLoc=xloc
	   px = c(.86,.86,xloc,xloc,NA)
	   polygon(px,pyThin,col=midCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
	if(lowCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),lowCut)$y
         lowCutLoc=xloc
	   px = c(.80,.80,xloc,xloc,NA)
	   polygon(px,pyThin,col=lowCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) slMinMax else slThresh,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) slMinMax else slThresh,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)   
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents	
leftEdge = ifelse(minVal < lowAdj,.14,.20)
rightEdge = ifelse(maxVal > hiAdj,.86,.80)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(slPer,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,slTitleY,"Percent of State-Five Year Periods",cex=cex,adj=.5)
text(.5,slLabY,sliderLabel[1],cex=cex,adj=.5)
par(xpd=TRUE)
lines(c(.1,.9),c(0,0),col="#E0E0E0")


#11 Emulated slider2 

tmpDat = as.numeric(mat1)	
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = thresholds1[1]
hiCut = thresholds1[2]

panelSelect(panelRows,5,1)
panelScale()

pyThick = c(slThickLow,slThickHi,slThickHi,slThickLow,NA)
pyThin  = c(slThinLow,slThinHi,slThinHi,slThinLow,NA)

if(minVal < lowAdj){
	px = c(.14,.14,.20,.20,NA)
	polygon(px,pyThin,col=lowCol1,border="black")
      text(.14,if(shift)slMinMax else slThresh,round(minVal,rnd),adj=.5,cex=cex)
	if(lowCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),lowCut)$y
         lowCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=midCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),hiCut)$y
         hiCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=hiCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
}

if(maxVal > hiAdj){
	px = c(.80,.80,.86,.86,NA)
	polygon(px,pyThin,col=hiCol1,border="black")
      text(.86,if(shift) slMinMax else slThresh,round(maxVal,rnd),adj=.5,cex=cex)
	if(hiCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),hiCut)$y
         hiCutLoc=xloc
	   px = c(.86,.86,xloc,xloc,NA)
	   polygon(px,pyThin,col=midCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
	if(lowCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),lowCut)$y
         lowCutLoc=xloc
	   px = c(.80,.80,xloc,xloc,NA)
	   polygon(px,pyThin,col=lowCol1)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol1)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) slMinMax else slThresh,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) slMinMax else slThresh,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol1)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)   
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol1)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents	
leftEdge = ifelse(minVal < lowAdj,.14,.20)
rightEdge = ifelse(maxVal > hiAdj,.86,.80)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(slPer,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,slTitleY,"Percent of All First Differences",cex=cex,adj=.5)
text(.5,slLabY,sliderLabel[2],cex=cex,adj=.5)



lines(c(.1,.9),c(0,0),col="#E0E0E0")



#12 Emulated slider3 

tmpDat = as.numeric(mat2)	
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = thresholds2[1]
hiCut = thresholds2[2]

panelSelect(panelRows,6,1)
panelScale()

pyThick = c(slThickLow,slThickHi,slThickHi,slThickLow,NA)
pyThin  = c(slThinLow,slThinHi,slThinHi,slThinLow,NA)

if(minVal < lowAdj){
	px = c(.14,.14,.20,.20,NA)
	polygon(px,pyThin,col=lowCol2,border="black")
      text(.14,if(shift)slMinMax else slThresh,round(minVal,rnd),adj=.5,cex=cex)
	if(lowCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),lowCut)$y
         lowCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=midCol2)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),hiCut)$y
         hiCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=hiCol2)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
}

if(maxVal > hiAdj){
	px = c(.80,.80,.86,.86,NA)
	polygon(px,pyThin,col=hiCol2,border="black")
      text(.86,if(shift) slMinMax else slThresh,round(maxVal,rnd),adj=.5,cex=cex)
	if(hiCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),hiCut)$y
         hiCutLoc=xloc
	   px = c(.86,.86,xloc,xloc,NA)
	   polygon(px,pyThin,col=midCol2)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)
      }
	if(lowCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),lowCut)$y
         lowCutLoc=xloc
	   px = c(.80,.80,xloc,xloc,NA)
	   polygon(px,pyThin,col=lowCol2)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol2)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) slMinMax else slThresh,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) slMinMax else slThresh,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol2)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(lowCut,rnd),adj=.5,cex=cex)   
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol2)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,slThresh,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents	
leftEdge = ifelse(minVal < lowAdj,.14,.20)
rightEdge = ifelse(maxVal > hiAdj,.86,.80)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(slPer,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,slTitleY,"Percent of All Second Differences",cex=cex,adj=.5)
text(.5,slLabY,sliderLabel[3],cex=cex,adj=.5)

if(!is.null(pdfFile))dev.off()

}


