TCmapsRiseFallLA = function(
         dat,
         border,
         borderAll=NULL,
         subs=1:ncol(dat),
         titles=c('title1','title2'),
         atTime=NULL,
         probCuts=c(.2,.8),
         thresholds=NULL,
         width=10,
         height=7.5,
         topMar = .4,
         rnd=0,
   sliderLabel="Age-Adjusted Deaths per 100,000",
         sliderPercents="Percent of Parish Years",
         cex=.9,
         xtext=.22,
         ytext=c(.80,.67,.50,.33,.20),
         pdfFile=NULL){


# ________________Script Sections__________________

#1  Class states by two thresholds for each time period  
#2  Page layout
#3  Colors and plotting parameters
#5  Plot title
#6  Outline panel groups
#7  Scaling for map panels
#8  Plot contours
#9 Emulated slider 

#1.  Class states by two thresholds for each time period_____________

dat = dat[,subs]
regionLabels=rownames(dat)
timeLabels=colnames(dat)
mat = as.matrix(dat)
matR = range(mat,na.rm=TRUE)

if(is.null(thresholds)){
thresholds = quantile(as.vector(mat),probs=probCuts,type=5,na.rm=TRUE)
}

breaks= c(matR[1],thresholds,matR[2])
matClass = matrix(cut(mat,breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat))
rownames(matClass) = regionLabels
matClass[is.na(matClass)]=4
polygonId = border$id[is.na(border$x)]

#2  Page Layout____________________________________

if(!is.null(pdfFile)){
     pdf(width=width,height=height,file=pdfFile)
   } else {
     windows(width=width,height=height)
}
topMar = topMar
bottomMar = .98
leftMar = 0
rightMar = 0
borders=rep(.2,4)
nc = ncol(dat)
nr=3

panels = panelLayout(nrow=nr,ncol=nc,borders=borders,
      topMar=topMar,bottomMar=bottomMar,
      leftMar=leftMar,rightMar=rightMar,
rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)

panelRows = panelLayout(nrow=nr,ncol=1,borders=borders,
      topMar=topMar,bottomMar=bottomMar,
      leftMar=leftMar,rightMar=rightMar,
rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)

colTabs  = panels$coltabs
colHalfWidth = diff(colTabs[2,])/2
rightPad = colHalfWidth
leftPad= colHalfWidth


panelShift =  panelLayout(nrow=nr,ncol=nc-1,borders=borders,
      topMar=topMar,bottomMar=bottomMar,
      leftMar=leftPad,rightMar=rightPad,
rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)

panelShiftRows =  panelLayout(nrow=nr,ncol=1,borders=borders,
      topMar=topMar,bottomMar=bottomMar,
      leftMar=leftPad,rightMar=rightPad,
rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)


panelTop  = panelLayout(nrow=1,ncol=1,borders=borders,
       topMar=topMar,bottomMar=bottomMar,
       leftMar=leftMar,rightMar=rightMar)

#3 Colors and plotting parameters_________________

oddCol =  "#E0FFFF"
evenCol = "#E0FFFF"
midGray = "#909090"

lightGray = "#E0E0E0"
lightYellow= "#FFFFA0"
backgroundColors = c(rep(lightGray,3),lightYellow)

foreColor = "#F00000"
foreLineCol = "#000000"
foreLineLwd = 1

lowCol = "#0080FF"
midCol = "#C0C0C0"      #  "#909090"   
hiCol = "#FF1010"
penNew = c(lowCol,midCol,hiCol)
penLeave = c("#80B0FF","#B0B0B0","#FFA0A0")
penSame = c("#80B0FF","#B0B0B0","#FFA0A0")

backColor = "#D0D0D0"
backLineCol = "#FFFFFF"
backLineLwd = 1

borderLineCol = "#303030"
borderLineLwd = 1
timeLine = .19
cex = cex

lightLineCol= "#B0B0B0"

#6  Plot title____________________________________

if(!is.null(titles)){
   par(xpd=TRUE)
   panelSelect(panelTop,mar='top')
   panelScale()

  if(length(titles)==1){
     text(.5,.8,titles[1],cex=1)
  } else {
    text(.5,.82,titles[1],cex=1) 
    text(.5,.62,titles[2],cex=1)
  }
}

#7 Outline upper and lower panels__________________________

panelSelect(panelRows,1,1)
panelScale()
panelFill(col=oddCol)
panelOutline(col=midGray)

panelSelect(panelRows,2,1)
panelScale()
panelFill(col=evenCol)
panelOutline(col=midGray)

panelSelect(panelRows,3,1)
panelScale()
panelFill(col=oddCol)
panelOutline(col=midGray)


#8 Scaling for map panels________________________________

#use specific data to 
#define border$id
#       border$x
#       border$y 



bndOrd = border$id[is.na(border$x)] # State abbrev
rxpoly = range(border$x,na.rm=T)
rxpoly = mean(rxpoly)+ 1.06*diff(rxpoly)*c(-.5,.5)
rypoly = range(border$y,na.rm=T)
rypoly = mean(rypoly)+ 1.08*diff(rypoly)*c(-.5,.5)

# 9 Plot Contours___________________________________________

panelSelect(panels,1,1)
panelScale()

text(rep(xtext,5),ytext,c("Rising","Parishes",
          "In","New","Color"),adj=.5,cex=cex)

panelSelect(panels,3,1)
panelScale()
text(rep(xtext,5),ytext,c("Falling","Parishes",
          "In","New","Color"),adj=.5,cex=cex)

for (j in 1:(nc-1)){
   # Scale the panel
   panelSelect(panelShift,1,j)   #
   panelScale(rxpoly,rypoly)
   # panelOutline()

   # plot states in with with gray outline
   polygon(border$x, border$y,
            col="white",border=lightLineCol) # fill states

   # changing regions 
  
   good = matClass[,j] < matClass[,j+1] 
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(border$id,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      penCol=penNew[matClass[forePolyId,j+1]]
      polygon(border$x[fore], border$y[fore],col=penCol,border=F) # fill states
      polygon(border$x[fore], border$y[fore],density=0,
         col=foreLineCol, lwd=1) # outline states
   }
}

for (j in 1:(nc-1)){
   # Scale the panel
   panelSelect(panelShift,3,j)   
   panelScale(rxpoly,rypoly)
   # panelOutline()

  # plot states in white with gray outline
   polygon(border$x, border$y,
            col="white",border=lightLineCol) # fill states

   # changing regions 
   good = matClass[,j] > matClass[,j+1] 
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(border$id,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      penCol=penNew[matClass[forePolyId,j+1]]
      polygon(border$x[fore], border$y[fore], col=penCol,border=F) # fill states
      polygon(border$x[fore], border$y[fore],density=0,
         col=foreLineCol, lwd=1) # outline states
   }
}

for(j in 1:nc){
   panelSelect(panels,1,j)
   panelScale()
   if(!is.null(atTime)){
      mtext(timeLabels[j],at=atTime,side=3,line=timeLine,font=2,cex=cex,col="black",adj=0)
   } else {
      mtext(timeLabels[j],side=3,line=timeLine,font=2,cex=cex,col="black") 
   }
}

for(j in 1:nc){
   panelSelect(panels,2,j)   #
   panelScale(rxpoly,rypoly)
   # plot states in gray with white outline
   polygon(border$x, border$y,col=backColor,border=T) # fill states
   polygon(border$x, border$y,density=0,
         col=backLineCol,lwd=backLineLwd) # outline states

   good = matClass[,j]==1 | matClass[,j]==3 
   if(any(good)){
       hRegions= regionLabels[good]
       fore = !is.na(match(border$id,hRegions))
       forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
       penCol=penNew[matClass[forePolyId,j]]
       polygon(border$x[fore], border$y[fore], col=penCol,border=F) # fill states
       polygon(border$x[fore], border$y[fore],density=0,
          col=foreLineCol, lwd=1) # outline states
   }
   # draw the US border
   if(!is.null(borderAll))
   polygon(borderAll$x,borderAll$y,density=0,
      col=borderLineCol,lwd=borderLineLwd) # outside boundary
}

#9 Emulated slider 

# small rectangles
#     between min and lower adjacent value 
#     between uppper adjacent value and max
# colored rectangles 
# lowCol between min and lower cut
#      hiCol beteween upper cut and max
#
#
# Small rectangles and color
#      If min < lower adjacent, plot in background color
#           plot min to min(lower cut and lower adjacent in blue)
#      if upper adjacent < max, plot in background color
#           plot max(uppercut and upper adjacent) to max in red      
#  
#  Large rectangles and color
#      plot lower to upper adjacent in background
#if (lower adjacent < the lower cut) plot in blue
#if (upper cut < upper adjacent) plot in red
#
#  Tick label locations 
#       lower and upper cut at locations just below thick rectangle
#       adjacent values ends of thick rectangle, shift down
#       If min < low adjacent end of tail rectangle shifted down
#          max > upper adjaent in of tail rectangle shifted down
#
#  Rectangle locations
#       adjacent values:       .20   .80 of panel width
#       min = lower adjacent     .14  .20 of panel width
#       max > upper adjacent    .80   .86 of panel width

tmpDat = as.numeric(as.matrix(dat))
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = thresholds[1]
hiCut = thresholds[2]

panelSelect(panels,mar='bottom')
panelScale()

pyThick = c(.35,.51,.51,.35,NA)
pyThin  = c(.39,.47,.47,.39,NA)

if(minVal < lowAdj){
px = c(.14,.14,.20,.20,NA)
polygon(px,pyThin,col=lowCol)
polygon(px,pyThin,col=1,density=0)
if(lowCut < lowAdj){
   xloc = approx(c(minVal,lowAdj),c(.14,.20),lowCut)$y
         lowCutLoc=xloc
   px = c(xloc,xloc,.20,.20,NA)
   polygon(px,pyThin,col=midCol)
   polygon(px,pyThin,col=1,density=0)
         lines(rep(xloc,2),pyThick[1:2],col='black')
   text(xloc,.26,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
   xloc = approx(c(minVal,lowAdj),c(.14,.20),hiCut)$y
         hiCutLoc=xloc
   px = c(xloc,xloc,.20,.20,NA)
   polygon(px,pyThin,col=hiCol)
   polygon(px,pyThin,col=1,density=0)
   text(xloc,.26,round(hiCut,rnd),adj=.5,cex=cex)
      }
      text(.14,.15,round(minVal,rnd),adj=.5,cex=cex)
}

if(maxVal > hiAdj){
px = c(.80,.80,.86,.86,NA)
polygon(px,pyThin,col=hiCol)
polygon(px,pyThin,col=1,density=0)
if(hiCut > hiAdj){
   xloc = approx(c(hiAdj,maxVal),c(.80,.86),hiCut)$y
         hiCutLoc=xloc
   px = c(.80,.80,xloc,xloc,NA)
   polygon(px,pyThin,col=midCol)
   polygon(px,pyThin,col=1,density=0)
         lines(rep(xloc,2),pyThick[1:2],col='black')
   text(xloc,.26,round(hiCut,rnd),adj=.5,cex=cex)
      }
if(lowCut > hiAdj){
   xloc = approx(c(hiAdj,maxVal),c(.80,.86),lowCut)$y
         lowCutLoc=xloc
   px = c(.80,.80,xloc,xloc,NA)
   polygon(px,pyThin,col=lowCol)
   polygon(px,pyThin,col=1,density=0)
   text(xloc,.26,round(lowCut,rnd),adj=.5,cex=cex)
      }
      text(.86,.15,round(maxVal,rnd),adj=.5,cex=cex)
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol)
  polygon(px,pyThick,col=1,density=0)
  text(.2,.14,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,.14,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,.26,round(lowCut,rnd),adj=.5,cex=cex)   
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,.26,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents
leftEdge = ifelse(minVal < lowAdj,.14,.20)
rightEdge = ifelse(maxVal > hiAdj,.86,.80)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(.62,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,.79,sliderPercents,cex=cex,adj=.5)
text(.5,.08,sliderLabel,cex=cex,adj=.5)

#panelSelect(panelTop,1,1)
#panelScale()
#panelOutline(col='#808080')
if(!is.null(pdfFile))dev.off()
}
