#          Comparative Micromaps: RiseFall Style
# Figure 6.6 White Female All Cancer
# Figure 6.7 Ratio of White Female to Male Comparative Mortality Ratios  

#            None standard so put last
# Figure 6.2 Homicide Rates Males 0 - 19     

#=========================================
#    Panel Function and State Boundary Files

source("panelFunctions.r")

stateVBorders = read.csv('stateVisibilityBorders.csv',
    row.names=NULL,header=TRUE,as.is=TRUE)
nationVBorders = read.csv('nationVisibilityBorders.csv',
    blank.lines.skip=F,row.names=NULL,header=TRUE,as.is=TRUE)      


#===========================================
#             RiseFall function
source("TCmapsRiseFall.r")


#____________Fig. 6.6___________________________

#     Read data and fix column names
wfAll2004 = as.matrix(read.csv("wfAll19502004.csv",row.names=1,as.is=TRUE))
colnames(wfAll2004) # verify
years = seq(1950,2000,by=5)
colnames(wfAll2004)= paste(years,"-",years+4,sep="")

#            Other Years    
#    The data starts in 1950
#    A longer series could be shown
#    There is missing data for Alaska and Hawaii
#    What happens could be checked 


#             Default Windows Device Version
TCmapsRiseFall(wfAll2004,subs=3:11,
titles="White Female Mortality Rates: All Cancers",
width=9.5,height=3.9,
topMar=0.2,rnd=0,timeLine=.05,
unitsLabel="Age-Adjusted Deaths Per 100,000",
percentsLabel="Percent of State-Years",
cex=.72,cexLegend=.60)

#             PDF device 

# f = 6.25/4.75 Poster enlargement
f=1
TCmapsRiseFall(wfAll2004,subs=3:11,
      titles="White Female Mortality Rates: All Cancers",
      width=f*9.5,height=f*3.9,
      topMar=f*0.2,botMar=f*.98,rnd=0,timeLine=.05,
      unitsLabel="Age-Adjusted Deaths Per 100,000",
      cex=f*.72,cexLegend=f*.60,
      pdfFile='Fig6_6_WFAllCancer.pdf')


#______________ Figure 6.7______________________________

#     Read data Mortalilty Ratio Data  and fix column names

wfToWmCompMort = as.matrix(read.csv("wfToWmCompMort.csv",
                 row.names=1,as.is=TRUE))
colnames(wfToWmCompMort) # verify
years = seq(1950,2000,by=5)
colnames(wfToWmCompMort)= paste(years,"-",years+4,sep="")

#             Windows Device

f=1
TCmapsRiseFall(wfToWmCompMort,subs=c(5,7,9,11),
      titles=c("Lung and Bronchus Cancer",
          "Ratio of White Female to White Male Comparative Mortality Ratios"),
      titleLines=c(.85,.34),width=f*6.5,height=f*5.2,
      topMar=f*0.4,rnd=2,timeLine=.05,
      unitsLabel="Ratio of Gender Specific Comparative Mortality Ratios",cex=.72,cexLegend=.60)

#              PDF Device
f=1
TCmapsRiseFall(wfToWmCompMort,subs=c(5,7,9,11),
      titles=c("Lung and Bronchus Cancer",
            "Ratio of White Female to White Male Comparative Mortality Ratios"),
      titleLines=c(.85,.34),
      width=f*6.5,height=f*5.2,
      topMar=f*0.4,rnd=2,timeLine=.05,
      unitsLabel="Ratio of Gender Specific Comparative Mortality Ratios",cex=.72,cexLegend=.60,
      pdfFile='Fig6_7wfToWmCompMortRatioLung.pdf')

#____________________Figure 6.2_________________________

#             RiseFall modified function
#               red text age groups
source("TCmapsRiseFallMod.r")

#     Read data and fix column names
wmHomAge = as.matrix(read.csv("maleHomicidesByAge.csv",row.names=1,as.is=TRUE))
colnames(wmHomAge) # verify
years = seq(-1,89,by=5)
tmp = paste("Ages ",years-4,"-",years,sep='')
tmp[1] = "Age 0"
tmp[2] = "Ages 1-4"
tmp[19] = "Ages 85+"
colnames(wmHomAge)= paste(tmp)

#              Windows Device  

TCmapsRiseFallMod(wmHomAge,subs=c(1,2,3,4,5),
                  titles=NULL,width=6.5,height=4.5,
                  topMar=0,rnd=2,
                  unitsLabel="Age Specific Deaths Per 100,000",cex=.75,
                  percentsLabel="Percent of State-Age Groups",
                  shift=list(minmaxLineUp=TRUE,unitsLabelDown=FALSE),
                  )


#                PDF

TCmapsRiseFallMod(wmHomAge,subs=c(1,2,3,4,5),
                  titles=NULL,width=6.5,height=4.5,
                  topMar=0,rnd=2,
                  unitsLabel="Age Specific Deaths Per 100,000",cex=.75,
                  percentsLabel="Percent of State-Age Groups",
                  shift=list(minmaxLineUp=TRUE,unitsLabelDown=FALSE),
                  pdfFile='Fig6_2_HomicideByAge.pdf')




