# File    TCmap2Way4Color.r
#
# By      Dan Carr
# Date    May 1,2009; Repackage May 1, 2010
#         Need argument Definitions
#         Emulated Slider not likely current

TCmap2Way4Color = function(
         dat1,
         dat2,
         subs=1:ncol(dat),
         titles=c('title1','title2'),
         titleLines = c(.82,.62),
         timeLine = .15,
         probCuts=c(.2,.8),
         threshholds = NULL,
         width=10,
         height=7.5,
         topMar = .4,
         rnd=0,
	   sliderLabel="Age-Adjusted Deaths per 100,000",
         sliderPercents="Percent of State Years",
         cex=.6,
         cexT = .8,
         borderCol="black",
         shift=TRUE,
         pdfFile=NULL,
         font2=1){


# ________________Script Sections__________________

#1  Class states by two threshholds for each time period  
#2  Page layout
#3  Colors and plotting parameters
#5  Plot title
#6  Outline panel groups
#7  Scaling for map panels
#8  Plot contours
#9 Emulated slider 

#1.  Class states by two threshholds for each time period_____________

dat1 = dat1[,subs]
regionLabels1=rownames(dat1)
timeLabels=colnames(dat1)
mat1 = as.matrix(dat1)

dat2 = dat2[,subs]
regionLabels2=rownames(dat2)
timeLabels=colnames(dat2)
if(!all.equal(regionLabels1,regionLabels2))stop('States in 2 file not aligned')
regionLabels = regionLabels1

mat2 = as.matrix(dat2)

mat = c(as.vector(mat1),as.vector(mat2))
matR = range(mat,na.rm=TRUE)
if(is.null(threshholds)){
   threshholds = quantile(as.vector(mat),probs=probCuts,type=5,na.rm=TRUE)
}
breaks= c(matR[1],threshholds,matR[2])
mat1Class = matrix(cut(mat1,breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat1))
mat2Class = matrix(cut(mat2,breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat2))
rownames(mat1Class) = regionLabels1
rownames(mat2Class) = regionLabels2

polygonId = stateVBorders$st[is.na(stateVBorders$x)]

#2  Page Layout____________________________________

if(!is.null(pdfFile)){
     pdf(width=width,height=height,file=pdfFile)
   } else {
     windows(width=width,height=height)
}

bottomMar = .98
leftMar = 0
rightMar = 0
borders=rep(.2,4)
nc = ncol(dat1)
nr=3

panels = panelLayout(nrow=nr,ncol=2*nc-1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1,colSep=c(.2,rep(0,2*nc-1)))

panelRows = panelLayout(nrow=nr,ncol=1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)
			
#3 Colors and plotting parameters_________________

oddCol =  "#E0FFFF"
evenCol = "#E0FFFF"
midGray = "#909090"

lightGray = "#E0E0E0"
lightYellow= "#FFFFA0"
backgroundColors = c(rep(lightGray,3),lightYellow)

foreColor = "#F00000"
foreLineCol = "#000000"
foreLineLwd = 1

lowCol = "#0080FF"
midCol = "#C0C0C0"      #  "#909090"   
hiCol = "#FF1010"
penNew = c(lowCol,midCol,hiCol)
penLeave = c("#80B0FF","#B0B0B0","#FFA0A0")
penSame = c("#80B0FF","#B0B0B0","#FFA0A0")
	
backColor = "#D0D0D0"
backLineCol = "#FFFFFF"
backLineLwd = 1

borderLineCol = "#A0A0A0"
borderLineLwd = 1

lightLineCol= "#B0B0B0"

lightYellowCol=  "#E0E0E0"   #  Yellow "#FFFFD0"    # Green "#D8FFD8" 

riseToMid = "#FFC0C0"
fallToMid = "#B0E0FF"

#6  Plot title____________________________________

if(!is.null(titles)){
   par(xpd=TRUE)
   panelSelect(panels,mar='top')
   panelScale()

  if(length(titles)==1){
     text(.5,titleLines[1],titles[1],cex=cexT)
  } else {
    text(.5,titleLines[1],titles[1],cex=cexT) 
    text(.5,titleLines[2],titles[2],cex=cexT)
  }
}


#8 Scaling for map panels________________________________

rxpoly = range(stateVBorders$x,na.rm=T)
rxpoly = mean(rxpoly)+ 1.06*diff(rxpoly)*c(-.5,.5)
rypoly = range(stateVBorders$y,na.rm=T)
rypoly = mean(rypoly)+ 1.08*diff(rypoly)*c(-.5,.5)

#9. Plot Contours___________________________________________

for(j in 1:nc){
   panelSelect(panels,1,2*j-1)
   panelScale()
   mtext(timeLabels[j],side=3,line=timeLine,cex=cex,font=font2)   #time Label
}

rowLab = c("Female","","Male")
for(i in 1:nr){
   panelSelect(panels,i,1)
   panelScale()
   mtext(rowLab[i],side=2,line=timeLine,cex=cex,font=font2)   #time Label
}


# First Row
for(j in 1:nc){
   panelSelect(panels,1,2*j-1) 
   panelScale(rxpoly,rypoly)
   panelFill(col=oddCol)
   panelOutline(col=borderLineCol)

   # plot states in gray with white outlines
   polygon(stateVBorders$x, stateVBorders$y,col=backColor,
      border="white",lwd=backLineLwd) # fill and outline
  
   # draw the US border
   polygon(nationVBorders$x,nationVBorders$y,density=0,
      col=borderCol,lwd=borderLineLwd) # outside boundary

   good = mat1Class[,j]==1 | mat1Class[,j]==3
   if(any(good)){
      hRegions= regionLabels1[good]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      penCol=penNew[mat1Class[forePolyId,j]]
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
}

# Second Row
for(j in 1:nc){
   panelSelect(panels,3,2*j-1) 
   panelScale(rxpoly,rypoly)
   panelFill(col=oddCol)
   panelOutline(col=borderLineCol)

   # plot states in gray with white outline
   polygon(stateVBorders$x, stateVBorders$y,col=backColor,
      border=backLineCol,lwd=backLineLwd) # fill and outline
  
   # draw the US border
   polygon(nationVBorders$x,nationVBorders$y,density=0,
      col=borderCol,lwd=borderLineLwd) # outside boundary

   good = mat2Class[,j]==1 | mat2Class[,j]==3
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      penCol=penNew[mat2Class[forePolyId,j]]
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
         border=foreLineCol) # fill states and outline
   }
}


#  Top Row Change
for(j in 1:(nc-1)){
   panelSelect(panels,1,2*j) 
   panelScale(rxpoly,rypoly)
   panelFill(col=lightYellowCol)
   polygon(stateVBorders$x, stateVBorders$y,
            col="white",border=lightLineCol) # fill states    

   dif = mat1Class[,j]!=mat1Class[,j+1]
   dif = ifelse(is.na(dif),FALSE,dif) 
   if(any(dif)){
      hRegions= regionLabels[dif]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      rise = mat1Class[dif,j] < mat1Class[dif,j+1]
      penNew = ifelse(rise,hiCol,lowCol)
      penNew = ifelse(rise & mat1Class[dif,j+1]==2,riseToMid,penNew)
      penNew = ifelse(!rise& mat1Class[dif,j+1]==2,fallToMid,penNew)
      penCol=penNew[forePolyId]
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
   panelOutline(col=borderLineCol)
}

#  Bottom Row Change
for(j in 1:(nc-1)){
   panelSelect(panels,3,2*j) 
   panelScale(rxpoly,rypoly)
   panelFill(col=lightYellowCol)
   polygon(stateVBorders$x, stateVBorders$y,
            col="white",border=lightLineCol) # fill states    
   dif = mat2Class[,j]!=mat2Class[,j+1]
   dif = ifelse(is.na(dif),FALSE,dif) 
   if(any(dif)){
      hRegions= regionLabels[dif]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      rise = mat2Class[dif,j] < mat2Class[dif,j+1]
      penNew = ifelse(rise,hiCol,lowCol)
      penNew = ifelse(rise & mat2Class[dif,j+1]==2,riseToMid,penNew)
      penNew = ifelse(!rise& mat2Class[dif,j+1]==2,fallToMid,penNew)
      penCol=penNew[forePolyId]  # linked by polygonId
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
   panelOutline(col=borderLineCol)
}

#  Group Change
for(j in 1:nc){
   panelSelect(panels,2,2*j-1) 
   panelScale(rxpoly,rypoly)
   panelFill(col=lightYellowCol)
   polygon(stateVBorders$x, stateVBorders$y,
            col="white",border=lightLineCol) # fill states    

   dif = mat1Class[,j]!=mat2Class[,j]
   dif = ifelse(is.na(dif),FALSE,dif) 
   if(any(dif)){
      hRegions= regionLabels[dif]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      rise = mat1Class[dif,j] < mat2Class[dif,j]
      penNew = ifelse(rise,hiCol,lowCol)
      penNew = ifelse(rise & mat2Class[dif,j]==2,riseToMid,penNew)
      penNew = ifelse(!rise& mat2Class[dif,j]==2,fallToMid,penNew)
      penCol=penNew[forePolyId]
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
  panelOutline(col=borderLineCol)
}


panelSelect(panels,2,2)
panelScale()
panelFill(lightYellowCol)
panelOutline(col=borderLineCol)
panelSelect(panels,2,4)
panelScale()
panelFill(lightYellowCol)
panelOutline(col=borderLineCol)


#9 Emulated slider 

# small rectangles
#     between min and lower adjacent value 
#     between uppper adjacent value and max
# colored rectangles 
#	 lowCol between min and lower cut
#      hiCol beteween upper cut and max
#
#
# Small rectangles and color
#      If min < lower adjacent, plot in background color
#           plot min to min(lower cut and lower adjacent in blue)
#      if upper adjacent < max, plot in background color
#           plot max(uppercut and upper adjacent) to max in red      
#  
#  Large rectangles and color
#      plot lower to upper adjacent in background
#		if (lower adjacent < the lower cut) plot in blue
#		if (upper cut < upper adjacent) plot in red
#
#  Tick label locations 
#       lower and upper cut at locations just below thick rectangle
#       adjacent values ends of thick rectangle, shift down
#       If min < low adjacent end of tail rectangle shifted down
#          max > upper adjaent in of tail rectangle shifted down	
#
#  Rectangle locations
#       adjacent values:    	   .20   .80 of panel width
#       min = lower adjacent     .14  .20 of panel width
#       max > upper adjacent    .80   .86 of panel width

tmpDat = mat	
minVal = min(tmpDat,na.rm=TRUE)
maxVal = max(tmpDat,na.rm=TRUE)
stats = boxplot(tmpDat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
lowCut = threshholds[1]
hiCut = threshholds[2]

panelSelect(panels,mar='bottom')
panelScale()

pyThick = c(.35,.51,.51,.35,NA)
pyThin  = c(.39,.47,.47,.39,NA)

if(minVal < lowAdj){
	px = c(.14,.14,.20,.20,NA)
	polygon(px,pyThin,col=lowCol)
	polygon(px,pyThin,col=1,density=0)
      text(.14,if(shift).15 else .26,round(minVal,rnd),adj=.5,cex=cex)
	if(lowCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),lowCut)$y
         lowCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=midCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,.26,round(lowCut,rnd),adj=.5,cex=cex)
      }
      if(hiCut < lowAdj){
	   xloc = approx(c(minVal,lowAdj),c(.14,.20),hiCut)$y
         hiCutLoc=xloc
	   px = c(xloc,xloc,.20,.20,NA)
	   polygon(px,pyThin,col=hiCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,.26,round(hiCut,rnd),adj=.5,cex=cex)
      }
}

if(maxVal > hiAdj){
	px = c(.80,.80,.86,.86,NA)
	polygon(px,pyThin,col=hiCol)
	polygon(px,pyThin,col=1,density=0)
      text(.86,if(shift) .15 else .26,round(maxVal,rnd),adj=.5,cex=cex)
	if(hiCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),hiCut)$y
         hiCutLoc=xloc
	   px = c(.86,.86,xloc,xloc,NA)
	   polygon(px,pyThin,col=midCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,.26,round(hiCut,rnd),adj=.5,cex=cex)
      }
	if(lowCut > hiAdj){
	   xloc = approx(c(hiAdj,maxVal),c(.80,.86),lowCut)$y
         lowCutLoc=xloc
	   px = c(.80,.80,xloc,xloc,NA)
	   polygon(px,pyThin,col=lowCol)
	   polygon(px,pyThin,col=1,density=0)
	   text(xloc,.26,round(lowCut,rnd),adj=.5,cex=cex)
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) .15 else .26,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) .15 else .26,round(hiAdj,rnd),adj=.5,cex=cex)

if(lowCut > lowAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),lowCut)$y
   lowCutLoc = xloc
   px = c(.2,.2,xloc,xloc,NA)
   polygon(px,pyThick,col=lowCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,.26,round(lowCut,rnd),adj=.5,cex=cex)   
}

if(hiCut < hiAdj){
   xloc = approx(c(lowAdj,hiAdj),c(.20,.80),hiCut)$y
   hiCutLoc=xloc
   px = c(xloc,xloc,.8,.8,NA)
   polygon(px,pyThick,col=hiCol)
   polygon(px,pyThick,col=1,density=0)
   text(xloc,.26,round(hiCut,rnd),adj=.5,cex=cex)   
}

# percents	
leftEdge = ifelse(minVal < lowAdj,.14,.20)
rightEdge = ifelse(maxVal > hiAdj,.86,.80)

classTot = table(matClass)
percent = round(100*classTot[1:3]/sum(classTot[1:3]),0)
percentLoc = c((leftEdge+lowCutLoc)/2,
                (lowCutLoc+hiCutLoc)/2,
                (hiCutLoc+rightEdge)/2)
text(percentLoc,rep(.62,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,.79,sliderPercents,cex=cex,adj=.5)
text(.5,.08,sliderLabel,cex=cex,adj=.5)
if(!is.null(pdfFile))dev.off()
}

