# File    TCmap2WayDif5Class.r
# By      Dan Carr

# Date    May 1,2009; Repackaged May 1,2010
# Need    Argument Description
#         Revise script and use more current
#             slider script/function 


TCmap2WayDiff5Class = function(
         dat1,
         dat2,
         subs=1:ncol(dat),
         titles=c('title1','title2'),
         titleLines = c(.82,.62),
         timeLine = .15,
         probCuts=c(.2,.4,.6,.8),
         threshholds = NULL,
         width=10,
         height=7.5,
         topMar = .4,
         rnd=0,
	   sliderLabel="Age-Adjusted Deaths per 100,000",
         sliderPercents="Percent of State-Years",
         cex=.6,
         cexT=.8,
         borderCol="black",
         shift=TRUE,
         pdfFile=NULL,
         font2=1,
         classCol=c("#008837","#A8D8B7",
                   "#FFFFFF","#C2A5CF",
                   "#7B3294")  )
{


# ________________Script Sections__________________

#1  Class states by two threshholds for each time period  
#2  Page layout
#3  Colors and plotting parameters
#5  Plot title
#6  Outline panel groups
#7  Scaling for map panels
#8  Plot contours
#9 Emulated slider 

#1.  Class states by two threshholds for each time period_____________

dat1 = dat1[,subs]
regionLabels1=rownames(dat1)
timeLabels=colnames(dat1)
mat1 = as.matrix(dat1)

dat2 = dat2[,subs]
regionLabels2=rownames(dat2)
timeLabels=colnames(dat2)
mat2 = as.matrix(dat2)
if(!all.equal(regionLabels1,regionLabels2))stop('States in 2 file not aligned')
regionLabels=regionLabels1

mat1dif = t(diff(t(mat1)))
mat2dif=  t(diff(t(mat2)))
matdif = mat1-mat2


# make color class matrices for differences over time
mat = c(as.vector(mat1dif),as.vector(mat2dif),as.vector(matdif))
matR = range(mat,rm.na=TRUE)


if(is.null(threshholds)){
   threshholds = quantile(as.vector(mat),probs=probCuts,type=5,na.rm=TRUE)
}
breaks= c(matR[1],threshholds,matR[2])
mat1difClass = matrix(cut(as.vector(mat1dif),breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat1dif))
mat2difClass = matrix(cut(as.vector(mat2dif),breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(mat2dif))

matdifClass = matrix(cut(as.vector(matdif),breaks=breaks,include.lowest=TRUE, labels=FALSE),
                  ncol=ncol(matdif))

rownames(mat1difClass) = regionLabels
rownames(mat2difClass) = regionLabels
rownames(matdifClass) = regionLabels

# make color class for differences over groups

polygonId = stateVBorders$st[is.na(stateVBorders$x)]

#2  Page Layout____________________________________

if(!is.null(pdfFile)){
     pdf(width=width,height=height,file=pdfFile)
   } else {
     windows(width=width,height=height)
}

bottomMar = .98
leftMar = 0
rightMar = 0
borders=rep(.2,4)
nc = ncol(dat1)
nr=3

panels = panelLayout(nrow=nr,ncol=2*nc-1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1,colSep=c(.2,rep(0,2*nc-1)))

panelRows = panelLayout(nrow=nr,ncol=1,borders=borders,
	      topMar=topMar,bottomMar=bottomMar,
	      leftMar=leftMar,rightMar=rightMar,
		rowSize=1,rowSep=c(.2,rep(0,nr)),
            colSize=1)
			
#3 Colors and plotting parameters_________________

oddCol =  "#E0FFFF"
evenCol = "#E0FFFF"
midGray = "#909090"

lightGray = "#E0E0E0"
lightYellow= "#FFFFA0"
backgroundColors = c(rep(lightGray,3),lightYellow)

foreColor = "#F00000"
foreLineCol = "#000000"
foreLineLwd = 1

lowCol = "#0080FF"
midCol = "white"      #  "#909090"   
hiCol = "#FF1010"
penNew = c(lowCol,midCol,hiCol)
penLeave = c("#80B0FF","#B0B0B0","#FFA0A0")
penSame = c("#80B0FF","#B0B0B0","#FFA0A0")
	

backLineCol = "#FFFFFF"
backLineLwd = 1

borderLineCol = "#A0A0A0"
borderLineLwd = 1


lightLineCol= "#B0B0B0"

lightYellowCol=  "#E0E0E0"   #  Yellow "#FFFFD0"    # Green "#D8FFD8" 

riseToMid = "#FFC0C0"
fallToMid = "#B0E0FF"


#6  Plot title____________________________________

if(!is.null(titles)){
   par(xpd=TRUE)
   panelSelect(panels,mar='top')
   panelScale()

  if(length(titles)==1){
     text(.5,titleLines[1],titles[1],cex=1)
  } else {
    text(.5,titleLines[1],titles[1],cex=1) 
    text(.5,titleLines[2],titles[2],cex=1)
  }
}


#8 Scaling for map panels________________________________

rxpoly = range(stateVBorders$x,na.rm=T)
rxpoly = mean(rxpoly)+ 1.06*diff(rxpoly)*c(-.5,.5)
rypoly = range(stateVBorders$y,na.rm=T)
rypoly = mean(rypoly)+ 1.08*diff(rypoly)*c(-.5,.5)

#9. Plot Contours___________________________________________

for(j in 1:nc){
   panelSelect(panels,1,2*j-1)
   panelScale()
   mtext(timeLabels[j],side=3,line=timeLine,cex=cex,font=font2)   #time Label
}

rowLab = c("Female","","Male")
for(i in 1:nr){
   panelSelect(panels,i,1)
   panelScale()
   mtext(rowLab[i],side=2,line=timeLine,cex=cex,font=font2)   #time Label
}


# First Row
for(j in 1:nc){
   panelSelect(panels,1,2*j-1) 
   panelScale(rxpoly,rypoly)
   panelFill(col=oddCol)
   panelOutline(col=borderLineCol)
}
#   polygon(stateVBorders$x, stateVBorders$y,
#            col="white",border=lightLineCol) # fill states
#
#   good = !is.na(mat1Class[,j])
#   if(any(good)){
#      hRegions= regionLabels[good]
#      fore = !is.na(match(stateVBorders$st,hRegions))
#      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
#      penCol=penNew[mat1Class[forePolyId,j]]
#      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
#      border=foreLineCol) # fill states and outline
#   }
#   panelOutline(col=borderLineCol)
#}

# Second Row
for(j in 1:nc){
   panelSelect(panels,3,2*j-1) 
   panelScale(rxpoly,rypoly)
   panelFill(col=oddCol)
   panelOutline(col=borderLineCol)
}
#   polygon(stateVBorders$x, stateVBorders$y,
#            col="white",border=lightLineCol) # fill states
#
#   good = !is.na(mat2Class[,j])
#   if(any(good)){
#      hRegions= regionLabels[good]
#      fore = !is.na(match(stateVBorders$st,hRegions))
#      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
#      penCol=penNew[mat2Class[forePolyId,j]]
#      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
#      border=foreLineCol) # fill states and outline
#   }
#  panelOutline(col=borderLineCol)
#}

#  Top Row Change
for(j in 1:(nc-1)){
   panelSelect(panels,1,2*j) 
   panelScale(rxpoly,rypoly)
   panelFill(col=lightYellowCol)
   polygon(stateVBorders$x, stateVBorders$y,
            col="white",border=lightLineCol) # fill states    

   good = mat1difClass[,j] != 3
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      penCol=classCol[mat1difClass[forePolyId,j]]
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
   panelOutline(col=borderLineCol)
}

#  Bottom Row Change
for(j in 1:(nc-1)){
   panelSelect(panels,3,2*j) 
   panelScale(rxpoly,rypoly)
   panelFill(col=lightYellowCol)
   polygon(stateVBorders$x, stateVBorders$y,
            col="white",border=lightLineCol) # fill states    
 
   good = mat2difClass[,j] !=3
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))] 
      penCol=classCol[mat2difClass[forePolyId,j]] 
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
   panelOutline(col=borderLineCol)
}

#  Group Change
for(j in 1:nc){
   panelSelect(panels,2,2*j-1) 
   panelScale(rxpoly,rypoly)
   panelFill(col=lightYellowCol)
   polygon(stateVBorders$x, stateVBorders$y,
            col="white",border=lightLineCol) # fill states    

   good = matdifClass[,j] !=3 
   if(any(good)){
      hRegions= regionLabels[good]
      fore = !is.na(match(stateVBorders$st,hRegions))
      forePolyId= polygonId[!is.na(match(polygonId,hRegions))]  
      penCol=classCol[matdifClass[forePolyId,j]]
      polygon(stateVBorders$x[fore], stateVBorders$y[fore],col=penCol,
      border=foreLineCol) # fill states and outline
   }
  panelOutline(col=borderLineCol)
}

panelSelect(panels,2,2)
panelScale()
panelFill(lightYellowCol)
panelOutline(col=borderLineCol)
panelSelect(panels,2,4)
panelScale()
panelFill(lightYellowCol)
panelOutline(col=borderLineCol)


#9 Emulated slider 

# small rectangles
#     between min and lower adjacent value 
#     between uppper adjacent value and max
# colored rectangles 
#	 lowCol between min and lower cut
#      hiCol beteween upper cut and max
#
#
# Small rectangles and color
#      If min < lower adjacent, plot in background color
#           plot min to min(lower cut and lower adjacent in blue)
#      if upper adjacent < max, plot in background color
#           plot max(uppercut and upper adjacent) to max in red      
#  
#  Large rectangles and color
#      plot lower to upper adjacent in background
#		if (lower adjacent < the lower cut) plot in blue
#		if (upper cut < upper adjacent) plot in red
#
#  Tick label locations 
#       lower and upper cut at locations just below thick rectangle
#       adjacent values ends of thick rectangle, shift down
#       If min < low adjacent end of tail rectangle shifted down
#          max > upper adjaent in of tail rectangle shifted down	
#
#  Rectangle locations
#       adjacent values:    	   .20   .80 of panel width
#       min = lower adjacent     .14  .20 of panel width
#       max > upper adjacent    .80   .86 of panel width
	
minVal = matR[1]
maxVal = matR[2]
stats = boxplot(mat,plot=F,na.rm=TRUE)$stats
lowAdj = stats[1]
hiAdj  = stats[5]
threshX = rep(0,length(threshholds))


panelSelect(panels,mar='bottom')
panelScale()

pyThick = c(.35,.51,.51,.35,NA)
pyThin  = c(.39,.47,.47,.39,NA)

if(minVal < lowAdj){
     px = c(.14,.14,.20,.20,NA)
     polygon(px,pyThin,col=colClass[1],border="black")
     text(.14,if(shift).15 else .26,round(minVal,rnd),adj=.5,cex=cex)
     for(i in 1:4){
	  if(threshholds[i] < lowAdj){
	     xloc = approx(c(minVal,lowAdj),c(.14,.20),threshholds[i])$y
	     px = c(xloc,xloc,.20,.20,NA)
	     polygon(px,pyThin,col=midCol)
	     polygon(px,pyThin,col=1,density=0)
	     text(xloc,.26,round(threshholds[i],rnd),adj=.5,cex=cex)
           threshX[i]=xloc
        }
     }
}

if(maxVal > hiAdj){
	px = c(.80,.80,.86,.86,NA)
	polygon(px,pyThin,col=colClass[5],border='black')
      text(.86,if(shift) .15 else .26,round(maxVal,rnd),adj=.5,cex=cex)
      for(i in 4:1){
	   if(threshholds[i] > hiAdj){
	      xloc = approx(c(hiAdj,maxVal),c(.80,.86),threshholds[i])$y
	      px = c(.86,.86,xloc,xloc,NA)
	      polygon(px,pyThin,col=colClass[i],border='black')
	      text(xloc,.26,round(threshholds[i],rnd),adj=.5,cex=cex)
            threshX[i]=xloc
         }
      }
}

# large Rectangles
  px = c(.20,.20,.80,.80,NA)
  polygon(px,pyThick,col=midCol)
  polygon(px,pyThick,col=1,density=0)
  text(.2,if(shift) .15 else .26,round(lowAdj,rnd),adj=.5,cex=cex)
  text(.8,if(shift) .15 else .26,round(hiAdj,rnd),adj=.5,cex=cex)

 xlow =.2 
 for(i in 1:4){
     if(threshholds[i]>lowAdj){ 
         xloc = approx(c(lowAdj,hiAdj),c(.20,.80),threshholds[i])$y
         xupper=min(xloc,.8)
         px = c(xlow,xlow,xupper,xupper,NA)
         polygon(px,pyThick,col=colClass[i],border="black")
         text(xupper,.26,round(threshholds[i],rnd),adj=.5,cex=cex)
         threshX[i]=xupper
         xlow=xupper
      }   
  }
  if(threshholds[4]<hiAdj){
         xloc = approx(c(lowAdj,hiAdj),c(.20,.80),threshholds[4])$y
         xupper=.8
         px = c(xloc,xloc,xupper,xupper,NA)
         polygon(px,pyThick,col=colClass[5],border="black")
  }
      

# percents	
leftEdge = ifelse(minVal < lowAdj,.14,.20)
rightEdge = ifelse(maxVal > hiAdj,.86,.80)

classTot = table(matdifClass)
percent = round(100*classTot[1:5]/sum(classTot[1:5]),0)
percentLoc = (c(leftEdge,threshX)+c(threshX,rightEdge))/2
text(percentLoc,rep(.62,3),paste(percent,'%',sep=''),cex=cex,adj=.5)
text(.5,.79,sliderPercents,cex=cex,adj=.5)
text(.5,.08,sliderLabel,cex=cex,adj=.5)
if(!is.null(pdfFile))dev.off()
}

